@@ -12,6 +12,8 @@
 # 2010-06-14: added wcrossprod and allow use of observation weights in Anova.mlm()
 # 2010-06-28: Fixed Anova() tables for coxph and survreg models 
 #             (failed because of changes in survival package.
+# 2011-01-21: Added functions for mixed models. J. Fox
+# 2011-01-25: Fixed Anova.polr() and Anova.multinom() to work with models with only one term. J. Fox
 #-------------------------------------------------------------------------------
 
 # Type II and III tests for linear, generalized linear, and other models (J. Fox)
@@ -441,7 +443,8 @@ Anova.II.multinom <- function (mod, ...)
 		rels <- names[relatives(names[term], names, fac)]
 		exclude.1 <- as.vector(unlist(sapply(c(names[term], rels),
 								which.nms)))
-		mod.1 <- multinom(y ~ X[, -c(1, exclude.1)], weights=wt, trace=FALSE)
+		mod.1 <-if (n.terms > 1) multinom(y ~ X[, -c(1, exclude.1)], weights=wt, trace=FALSE)
+				else multinom(y ~ 1, weights=wt, race=FALSE)
 		dev.1 <- deviance(mod.1)
 		mod.2 <- if (length(rels) == 0)
 					mod
@@ -475,7 +478,8 @@ Anova.III.multinom <- function (mod, ...)
 	df <- df.terms(mod)
 	deviance <- deviance(mod)
 	for (term in 1:n.terms) {
-		mod.1 <- multinom(y ~ X[, term != asgn][, -1], weights=wt, trace=FALSE)
+		mod.1 <- if (n.terms > 1) multinom(y ~ X[, term != asgn][, -1], weights=wt, trace=FALSE)
+				else multinom(y ~ 1, weights=wt, trace=FALSE)
 		LR[term] <- deviance(mod.1) - deviance
 		p[term] <- pchisq(LR[term], df[term], lower.tail=FALSE)
 	}
@@ -524,7 +528,8 @@ Anova.II.polr <- function (mod, ...)
 		rels <- names[relatives(names[term], names, fac)]
 		exclude.1 <- as.vector(unlist(sapply(c(names[term], rels),
 								which.nms)))
-		mod.1 <- polr(y ~ X[, -c(1, exclude.1)], weights=wt)
+		mod.1 <- if (n.terms > 1) polr(y ~ X[, -c(1, exclude.1)], weights=wt)
+				else polr(y ~ 1, weights=wt)
 		dev.1 <- deviance(mod.1)
 		mod.2 <- if (length(rels) == 0)
 					mod
@@ -557,7 +562,8 @@ Anova.III.polr <- function (mod, ...)
 	df <- df.terms(mod)
 	deviance <- deviance(mod)
 	for (term in 1:n.terms) {
-		mod.1 <- polr(y ~ X[, term != asgn][, -1], weights=wt)
+		mod.1 <- if (n.terms > 1) polr(y ~ X[, term != asgn][, -1], weights=wt)
+				else polr(y ~ 1, weights=wt)
 		LR[term] <- deviance(mod.1) - deviance
 		p[term] <- pchisq(LR[term], df[term], lower.tail=FALSE)
 	}
@@ -1433,3 +1439,211 @@ Anova.III.default <- function(mod, vcov., test, singular.ok=FALSE, ...){
 			paste("Response:", responseName(mod)))
 	result
 }
+
+## functions for mixed models
+
+Anova.mer <- function(mod, type=c("II","III", 2, 3),
+		vcov.=vcov(mod), singular.ok, ...){
+	type <- as.character(type)
+	type <- match.arg(type)
+	if (missing(singular.ok))
+		singular.ok <- type == "2" || type == "II"
+	switch(type,
+			II=Anova.II.mer(mod, vcov., singular.ok=singular.ok),
+			III=Anova.III.mer(mod, vcov., singular.ok=singular.ok),
+			"2"=Anova.II.mer(mod, vcov., singular.ok=singular.ok),
+			"3"=Anova.III.mer(mod, vcov., singular.ok=singular.ok))
+}
+
+Anova.II.mer <- function(mod, vcov., singular.ok=TRUE, ...){
+	hyp.term <- function(term){
+		which.term <- which(term==names)
+		subs.term <- which(assign==which.term)
+		relatives <- relatives(term, names, fac)
+		subs.relatives <- NULL
+		for (relative in relatives) 
+			subs.relatives <- c(subs.relatives, which(assign==relative))
+		hyp.matrix.1 <- I.p[subs.relatives,,drop=FALSE]
+		hyp.matrix.1 <- hyp.matrix.1[, not.aliased, drop=FALSE]
+		hyp.matrix.2 <- I.p[c(subs.relatives,subs.term),,drop=FALSE]
+		hyp.matrix.2 <- hyp.matrix.2[, not.aliased, drop=FALSE]       
+		hyp.matrix.term <- if (nrow(hyp.matrix.1) == 0) hyp.matrix.2
+				else t(ConjComp(t(hyp.matrix.1), t(hyp.matrix.2), vcov.))            
+		hyp.matrix.term <- hyp.matrix.term[!apply(hyp.matrix.term, 1, 
+						function(x) all(x == 0)), , drop=FALSE]
+		if (nrow(hyp.matrix.term) == 0)
+			return(c(statistic=NA, df=0))            
+		hyp <- linearHypothesis(mod, hyp.matrix.term, 
+				vcov.=vcov., singular.ok=singular.ok, ...)
+		c(statistic=hyp$Chisq[2], df=hyp$Df[2])
+	}
+	not.aliased <- !is.na(fixef(mod))
+	if (!singular.ok && !all(not.aliased))
+		stop("there are aliased coefficients in the model")
+	fac <- attr(terms(mod), "factors")
+	intercept <- has.intercept(mod)
+	p <- length(fixef(mod))
+	I.p <- diag(p)
+	assign <- attr(model.matrix(mod), "assign")
+	assign[!not.aliased] <- NA
+	names <- term.names(mod)
+	if (intercept) names <- names[-1]
+	n.terms <- length(names)
+	p <- teststat <- df <- rep(0, n.terms)
+	for (i in 1:n.terms){
+		hyp <- hyp.term(names[i])
+		teststat[i] <- abs(hyp["statistic"])
+		df[i] <- abs(hyp["df"])
+		p[i] <- pchisq(teststat[i], df[i], lower.tail=FALSE) 
+	}    
+	result <- data.frame(df, teststat, p)
+	row.names(result) <- names
+	names(result) <- c ("Df", "Chisq", "Pr(>Chisq)")
+	class(result) <- c("anova", "data.frame")
+	attr(result, "heading") <- c("Analysis of Deviance Table (Type II tests)\n", 
+			paste("Response:", responseName(mod)))
+	result
+}
+
+Anova.III.mer <- function(mod, vcov., singular.ok=FALSE, ...){
+	intercept <- has.intercept(mod)
+	p <- length(fixef(mod))
+	I.p <- diag(p)
+	names <- term.names(mod)
+	n.terms <- length(names)
+	assign <- attr(model.matrix(mod), "assign")
+	df <- rep(0, n.terms)
+	if (intercept) df[1] <- 1
+	p <- teststat <- rep(0, n.terms)
+	not.aliased <- !is.na(fixef(mod))
+	if (!singular.ok && !all(not.aliased))
+		stop("there are aliased coefficients in the model")
+	for (term in 1:n.terms){
+		subs <- which(assign == term - intercept)        
+		hyp.matrix <- I.p[subs,,drop=FALSE]
+		hyp.matrix <- hyp.matrix[, not.aliased, drop=FALSE]
+		hyp.matrix <- hyp.matrix[!apply(hyp.matrix, 1, function(x) all(x == 0)), , drop=FALSE]        
+		if (nrow(hyp.matrix) == 0){
+			teststat[term] <- NA
+			df[term] <- 0
+			p[term] <- NA
+		}
+		else {
+			hyp <- linearHypothesis(mod, hyp.matrix, 
+					vcov.=vcov., singular.ok=singular.ok, ...)
+			teststat[term] <-  hyp$Chisq[2] 
+			df[term] <- abs(hyp$Df[2])
+			p[term] <- pchisq(teststat[term], df[term], lower.tail=FALSE) 
+		}
+	}
+	result <- data.frame(df, teststat, p)
+	row.names(result) <- names
+	names(result) <- c ("Df", "Chisq",  "Pr(>Chisq)")
+	class(result) <- c("anova", "data.frame")
+	attr(result, "heading") <- c("Analysis of Deviance Table (Type III tests)\n", 
+			paste("Response:", responseName(mod)))
+	result
+}
+
+Anova.lme <- function(mod, type=c("II","III", 2, 3),
+		vcov.=vcov(mod), singular.ok, ...){
+	type <- as.character(type)
+	type <- match.arg(type)
+	if (missing(singular.ok))
+		singular.ok <- type == "2" || type == "II"
+	switch(type,
+			II=Anova.II.lme(mod, vcov., singular.ok=singular.ok),
+			III=Anova.III.lme(mod, vcov., singular.ok=singular.ok),
+			"2"=Anova.II.lme(mod, vcov., singular.ok=singular.ok),
+			"3"=Anova.III.lme(mod, vcov., singular.ok=singular.ok))
+}
+
+Anova.II.lme <- function(mod, vcov., singular.ok=TRUE, ...){
+	hyp.term <- function(term){
+		which.term <- which(term==names)
+		subs.term <- which(assign==which.term)
+		relatives <- relatives(term, names, fac)
+		subs.relatives <- NULL
+		for (relative in relatives) 
+			subs.relatives <- c(subs.relatives, which(assign==relative))
+		hyp.matrix.1 <- I.p[subs.relatives,,drop=FALSE]
+		hyp.matrix.1 <- hyp.matrix.1[, not.aliased, drop=FALSE]
+		hyp.matrix.2 <- I.p[c(subs.relatives,subs.term),,drop=FALSE]
+		hyp.matrix.2 <- hyp.matrix.2[, not.aliased, drop=FALSE]       
+		hyp.matrix.term <- if (nrow(hyp.matrix.1) == 0) hyp.matrix.2
+				else t(ConjComp(t(hyp.matrix.1), t(hyp.matrix.2), vcov.))            
+		hyp.matrix.term <- hyp.matrix.term[!apply(hyp.matrix.term, 1, 
+						function(x) all(x == 0)), , drop=FALSE]
+		if (nrow(hyp.matrix.term) == 0)
+			return(c(statistic=NA, df=0))            
+		hyp <- linearHypothesis(mod, hyp.matrix.term, 
+				vcov.=vcov., singular.ok=singular.ok, ...)
+		c(statistic=hyp$Chisq[2], df=hyp$Df[2])
+	}
+	not.aliased <- !is.na(fixef(mod))
+	if (!singular.ok && !all(not.aliased))
+		stop("there are aliased coefficients in the model")
+	fac <- attr(terms(mod), "factors")
+	intercept <- has.intercept(mod)
+	p <- length(fixef(mod))
+	I.p <- diag(p)
+	assign <- attr(model.matrix(mod), "assign")
+	assign[!not.aliased] <- NA
+	names <- term.names(mod)
+	if (intercept) names <- names[-1]
+	n.terms <- length(names)
+	p <- teststat <- df <- rep(0, n.terms)
+	for (i in 1:n.terms){
+		hyp <- hyp.term(names[i])
+		teststat[i] <- abs(hyp["statistic"])
+		df[i] <- abs(hyp["df"])
+		p[i] <- pchisq(teststat[i], df[i], lower.tail=FALSE) 
+	}    
+	result <- data.frame(df, teststat, p)
+	row.names(result) <- names
+	names(result) <- c ("Df", "Chisq", "Pr(>Chisq)")
+	class(result) <- c("anova", "data.frame")
+	attr(result, "heading") <- c("Analysis of Deviance Table (Type II tests)\n", 
+			paste("Response:", responseName(mod)))
+	result
+}
+
+Anova.III.lme <- function(mod, vcov., singular.ok=FALSE, ...){
+	intercept <- has.intercept(mod)
+	p <- length(coefficients(mod))
+	I.p <- diag(p)
+	names <- term.names(mod)
+	n.terms <- length(names)
+	assign <- attr(model.matrix(mod), "assign")
+	df <- rep(0, n.terms)
+	if (intercept) df[1] <- 1
+	p <- teststat <-rep(0, n.terms)
+	not.aliased <- !is.na(fixef(mod))
+	if (!singular.ok && !all(not.aliased))
+		stop("there are aliased coefficients in the model")
+	for (term in 1:n.terms){
+		subs <- which(assign == term - intercept)        
+		hyp.matrix <- I.p[subs,,drop=FALSE]
+		hyp.matrix <- hyp.matrix[, not.aliased, drop=FALSE]
+		hyp.matrix <- hyp.matrix[!apply(hyp.matrix, 1, function(x) all(x == 0)), , drop=FALSE]        
+		if (nrow(hyp.matrix) == 0){
+			teststat[term] <- NA
+			df[term] <- 0
+			p[term] <- NA
+		}
+		else {
+			hyp <- linearHypothesis(mod, hyp.matrix, 
+					vcov.=vcov., singular.ok=singular.ok, ...)
+			teststat[term] <-  hyp$Chisq[2] 
+			df[term] <- abs(hyp$Df[2])
+			p[term] <- pchisq(teststat[term], df[term], lower.tail=FALSE) 
+		}
+	}
+	result <- data.frame(df, teststat, p)
+	row.names(result) <- names
+	names(result) <- c ("Df", "Chisq",  "Pr(>Chisq)")
+	class(result) <- c("anova", "data.frame")
+	attr(result, "heading") <- c("Analysis of Deviance Table (Type III tests)\n", 
+			paste("Response:", responseName(mod)))
+	result
+}
